"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const { dirname, basename } = require('./path-utils');
function removeTrailingColon(path) {
    if (!path || !path.length)
        return '';
    if (path[path.length - 1] === ':')
        return path.substr(0, path.length - 1);
    return path;
}
// All input paths should be in the format: "SPECIAL_DIR:/path/to/file"
// The trailing colon must not be included as it's automatically added
// when doing the API call.
// Only supported special dir at the moment is "root"
class FileApiDriverJoplinServer {
    constructor(api) {
        this.api_ = api;
    }
    initialize(basePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const pieces = removeTrailingColon(basePath).split('/');
            if (!pieces.length)
                return;
            let parent = pieces.splice(0, 1)[0];
            for (const p of pieces) {
                // Syncing with the root, which is ok, and in that
                // case there's no sub-dir to create.
                if (!p && pieces.length === 1)
                    return;
                const subPath = `${parent}/${p}`;
                yield this.mkdir(subPath);
                parent = subPath;
            }
        });
    }
    api() {
        return this.api_;
    }
    requestRepeatCount() {
        return 3;
    }
    metadataToStat_(md, path, isDeleted = false) {
        const output = {
            path: path,
            updated_time: md.updated_time,
            isDir: !!md.is_directory,
            isDeleted: isDeleted,
        };
        // TODO - HANDLE DELETED
        // if (md['.tag'] === 'deleted') output.isDeleted = true;
        return output;
    }
    metadataToStats_(mds) {
        const output = [];
        for (let i = 0; i < mds.length; i++) {
            output.push(this.metadataToStat_(mds[i], mds[i].name));
        }
        return output;
    }
    apiFilePath_(p) {
        if (p !== 'root')
            p += ':';
        return `api/files/${p}`;
    }
    stat(path) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.api().exec('GET', this.apiFilePath_(path));
                return this.metadataToStat_(response, path);
            }
            catch (error) {
                if (error.code === 404)
                    return null;
                throw error;
            }
        });
    }
    delta(path, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const context = options ? options.context : null;
            let cursor = context ? context.cursor : null;
            while (true) {
                try {
                    const query = cursor ? { cursor } : {};
                    const response = yield this.api().exec('GET', `${this.apiFilePath_(path)}/delta`, query);
                    const stats = response.items.map((item) => {
                        return this.metadataToStat_(item.item, item.item.name, item.type === 3);
                    });
                    const output = {
                        items: stats,
                        hasMore: response.has_more,
                        context: { cursor: response.cursor },
                    };
                    return output;
                }
                catch (error) {
                    // If there's an error related to an invalid cursor, clear the cursor and retry.
                    if (cursor && error.code === 'resyncRequired') {
                        cursor = null;
                        continue;
                    }
                    throw error;
                }
            }
        });
    }
    list(path, options = null) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            options = Object.assign({ context: null }, options);
            const query = ((_a = options.context) === null || _a === void 0 ? void 0 : _a.cursor) ? { cursor: options.context.cursor } : null;
            const results = yield this.api().exec('GET', `${this.apiFilePath_(path)}/children`, query);
            const newContext = {};
            if (results.cursor)
                newContext.cursor = results.cursor;
            return {
                items: this.metadataToStats_(results.items),
                hasMore: results.has_more,
                context: newContext,
            };
        });
    }
    get(path, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!options)
                options = {};
            if (!options.responseFormat)
                options.responseFormat = 'text';
            try {
                const response = yield this.api().exec('GET', `${this.apiFilePath_(path)}/content`, null, null, null, options);
                return response;
            }
            catch (error) {
                if (error.code !== 404)
                    throw error;
                return null;
            }
        });
    }
    parentPath_(path) {
        let output = dirname(path);
        // This is the root or a special folder
        if (output.split('/').length === 1) {
            output = output.substr(0, output.length - 1);
        }
        return output;
    }
    basename_(path) {
        return basename(path);
    }
    mkdir(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const parentPath = this.parentPath_(path);
            const filename = this.basename_(path);
            try {
                const response = yield this.api().exec('POST', `${this.apiFilePath_(parentPath)}/children`, null, {
                    name: filename,
                    is_directory: 1,
                });
                return response;
            }
            catch (error) {
                // 409 is OK - directory already exists
                if (error.code !== 409)
                    throw error;
            }
        });
    }
    put(path, content, options = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.api().exec('PUT', `${this.apiFilePath_(path)}/content`, null, content, {
                'Content-Type': 'application/octet-stream',
            }, options);
        });
    }
    delete(path) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.api().exec('DELETE', this.apiFilePath_(path));
        });
    }
    format() {
        throw new Error('Not supported');
    }
    clearRoot(path) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.delete(path);
            yield this.mkdir(path);
        });
    }
}
exports.default = FileApiDriverJoplinServer;
//# sourceMappingURL=file-api-driver-joplinServer.js.map